@@ -10,10 +10,6 @@ from mloda_core.abstract_plugins.abstract_feature_group import AbstractFeatureGr
 from mloda_core.abstract_plugins.components.base_artifact import BaseArtifact
 from mloda_core.abstract_plugins.components.feature import Feature
 from mloda_core.abstract_plugins.components.feature_chainer.feature_chain_parser import FeatureChainParser
-from mloda_core.abstract_plugins.components.feature_chainer.feature_chainer_parser_configuration import (
-    FeatureChainParserConfiguration,
-    create_configurable_parser,
-)
 from mloda_core.abstract_plugins.components.feature_name import FeatureName
 from mloda_core.abstract_plugins.components.feature_set import FeatureSet
 from mloda_core.abstract_plugins.components.options import Options
@@ -22,51 +18,60 @@ from mloda_plugins.feature_group.experimental.forecasting.forecasting_artifact i
 
 
 class ForecastingFeatureGroup(AbstractFeatureGroup):
-    # Option keys for forecasting configuration
-    ALGORITHM = "algorithm"
-    HORIZON = "horizon"
-    TIME_UNIT = "time_unit"
     """
     Base class for all forecasting feature groups.
 
     Forecasting feature groups generate forecasts for time series data using various algorithms.
     They allow you to predict future values based on historical patterns and trends.
+    Supports both string-based feature creation and configuration-based creation with proper
+    group/context parameter separation.
 
-    ## Feature Naming Convention
+    ## Feature Creation Methods
 
-    Forecasting features follow this naming pattern:
-    `{algorithm}_forecast_{horizon}{time_unit}__{mloda_source_feature}`
+    ### 1. String-Based Creation
 
-    The source feature (mloda_source_feature) is extracted from the feature name and used
-    as input for the forecasting algorithm. Note the double underscore before the source feature.
+    Features follow the naming pattern: `{algorithm}_forecast_{horizon}{time_unit}__{mloda_source_feature}`
 
     Examples:
-    - `linear_forecast_7day__sales`: 7-day forecast of sales using linear regression
-    - `randomforest_forecast_24hr__energy_consumption`: 24-hour forecast of energy consumption using random forest
-    - `svr_forecast_3month__demand`: 3-month forecast of demand using support vector regression
+    ```python
+    features = [
+        "linear_forecast_7day__sales",      # 7-day forecast of sales using linear regression
+        "randomforest_forecast_24hr__energy_consumption",  # 24-hour forecast using random forest
+        "svr_forecast_3month__demand"       # 3-month forecast using support vector regression
+    ]
+    ```
 
-    ## Configuration-Based Creation
+    ### 2. Configuration-Based Creation
 
-    ForecastingFeatureGroup supports configuration-based creation through the
-    FeatureChainParserConfiguration mechanism. This allows features to be created
-    from options rather than explicit feature names.
-
-    To create a forecasting feature using configuration:
+    Uses Options with proper group/context parameter separation:
 
     ```python
     feature = Feature(
-        "PlaceHolder",  # Placeholder name, will be replaced
-        Options({
-            ForecastingFeatureGroup.ALGORITHM: "linear",
-            ForecastingFeatureGroup.HORIZON: 7,
-            ForecastingFeatureGroup.TIME_UNIT: "day",
-            DefaultOptionKeys.mloda_source_feature: "sales"
-        })
+        name="placeholder",
+        options=Options(
+            context={
+                ForecastingFeatureGroup.ALGORITHM: "linear",
+                ForecastingFeatureGroup.HORIZON: 7,
+                ForecastingFeatureGroup.TIME_UNIT: "day",
+                DefaultOptionKeys.mloda_source_feature: "sales",
+            }
+        )
     )
-
-    # The Engine will automatically parse this into a feature with name "linear_forecast_7day__sales"
     ```
 
+    ## Parameter Classification
+
+    ### Context Parameters (Default)
+    These parameters don't affect Feature Group resolution/splitting:
+    - `algorithm`: The forecasting algorithm to use
+    - `horizon`: The forecast horizon (number of time units)
+    - `time_unit`: The time unit for the horizon
+    - `mloda_source_feature`: The source feature to generate forecasts for
+
+    ### Group Parameters
+    Currently none for ForecastingFeatureGroup. Parameters that affect Feature Group
+    resolution/splitting would be placed here.
+
     ## Supported Forecasting Algorithms
 
     - `linear`: Linear regression
@@ -93,6 +98,11 @@ class ForecastingFeatureGroup(AbstractFeatureGroup):
     - You can specify a custom time column by setting the reference_time option in the feature group options
     """
 
+    # Option keys for forecasting configuration
+    ALGORITHM = "algorithm"
+    HORIZON = "horizon"
+    TIME_UNIT = "time_unit"
+
     # Define supported forecasting algorithms
     FORECASTING_ALGORITHMS = {
         "linear": "Linear Regression",
@@ -117,6 +127,35 @@ class ForecastingFeatureGroup(AbstractFeatureGroup):
 
     # Define the prefix pattern for this feature group
     PREFIX_PATTERN = r"^([\w]+)_forecast_(\d+)([\w]+)__"
+    PATTERN = "__"
+
+    # Property mapping for configuration-based features with group/context separation
+    PROPERTY_MAPPING = {
+        ALGORITHM: {
+            **FORECASTING_ALGORITHMS,
+            DefaultOptionKeys.mloda_context: True,
+            DefaultOptionKeys.mloda_strict_validation: True,
+        },
+        HORIZON: {
+            "explanation": "Forecast horizon (number of time units to predict)",
+            DefaultOptionKeys.mloda_context: True,
+            DefaultOptionKeys.mloda_strict_validation: True,
+            DefaultOptionKeys.mloda_validation_function: lambda x: (
+                isinstance(x, int) or (isinstance(x, str) and x.isdigit())
+            )
+            and int(x) > 0,
+        },
+        TIME_UNIT: {
+            **TIME_UNITS,
+            DefaultOptionKeys.mloda_context: True,
+            DefaultOptionKeys.mloda_strict_validation: True,
+        },
+        DefaultOptionKeys.mloda_source_feature: {
+            "explanation": "Source feature to generate forecasts for",
+            DefaultOptionKeys.mloda_context: True,
+            DefaultOptionKeys.mloda_strict_validation: False,
+        },
+    }
 
     @staticmethod
     def artifact() -> Type[BaseArtifact] | None:
@@ -150,19 +189,26 @@ class ForecastingFeatureGroup(AbstractFeatureGroup):
         return reference_time_key
 
     def input_features(self, options: Options, feature_name: FeatureName) -> Optional[Set[Feature]]:
-        """
-        Extract source feature and time filter feature from the feature name.
+        """Extract source feature and time filter feature from either configuration-based options or string parsing."""
 
-        Args:
-            options: The options for the feature
-            feature_name: The name of the feature
+        source_feature: str | None = None
 
-        Returns:
-            A set containing the source feature and time filter feature
-        """
-        source_feature = FeatureChainParser.extract_source_feature(feature_name.name, self.PREFIX_PATTERN)
+        # Try string-based parsing first
+        _, source_feature = FeatureChainParser.parse_feature_name(feature_name, self.PATTERN, [self.PREFIX_PATTERN])
+        if source_feature is not None:
+            time_filter_feature = Feature(self.get_time_filter_feature(options))
+            return {Feature(source_feature), time_filter_feature}
+
+        # Fall back to configuration-based approach
+        source_features = options.get_source_features()
+        if len(source_features) != 1:
+            raise ValueError(
+                f"Expected exactly one source feature, but found {len(source_features)}: {source_features}"
+            )
+
+        source_feature_obj = next(iter(source_features))
         time_filter_feature = Feature(self.get_time_filter_feature(options))
-        return {Feature(source_feature), time_filter_feature}
+        return {source_feature_obj, time_filter_feature}
 
     @classmethod
     def parse_forecast_prefix(cls, feature_name: str) -> tuple[str, int, str]:
@@ -229,21 +275,6 @@ class ForecastingFeatureGroup(AbstractFeatureGroup):
 
         return algorithm, horizon, time_unit
 
-    @classmethod
-    def get_algorithm(cls, feature_name: str) -> str:
-        """Extract the forecasting algorithm from the feature name."""
-        return cls.parse_forecast_prefix(feature_name)[0]
-
-    @classmethod
-    def get_horizon(cls, feature_name: str) -> int:
-        """Extract the forecast horizon from the feature name."""
-        return cls.parse_forecast_prefix(feature_name)[1]
-
-    @classmethod
-    def get_time_unit(cls, feature_name: str) -> str:
-        """Extract the time unit from the feature name."""
-        return cls.parse_forecast_prefix(feature_name)[2]
-
     @classmethod
     def match_feature_group_criteria(
         cls,
@@ -252,19 +283,15 @@ class ForecastingFeatureGroup(AbstractFeatureGroup):
         data_access_collection: Optional[Any] = None,
     ) -> bool:
         """Check if feature name matches the expected pattern for forecasting features."""
-        if isinstance(feature_name, FeatureName):
-            feature_name = feature_name.name
 
-        try:
-            # First validate that this is a valid feature name for this feature group
-            if not FeatureChainParser.validate_feature_name(feature_name, cls.PREFIX_PATTERN):
-                return False
-
-            # Then validate the forecasting components
-            cls.parse_forecast_prefix(feature_name)
-            return True
-        except ValueError:
-            return False
+        # Use the unified parser with property mapping for full configuration support
+        return FeatureChainParser.match_configuration_feature_chain_parser(
+            feature_name,
+            options,
+            property_mapping=cls.PROPERTY_MAPPING,
+            pattern=cls.PATTERN,
+            prefix_patterns=[cls.PREFIX_PATTERN],
+        )
 
     @classmethod
     def calculate_feature(cls, data: Any, features: FeatureSet) -> Any:
@@ -272,24 +299,38 @@ class ForecastingFeatureGroup(AbstractFeatureGroup):
         Perform forecasting operations.
 
         Processes all requested features, determining the forecasting algorithm,
-        horizon, time unit, and source feature from each feature name.
+        horizon, time unit, and source feature from either string parsing or
+        configuration-based options.
 
         If a trained model exists in the artifact, it is used to generate forecasts.
         Otherwise, a new model is trained and saved as an artifact.
 
         Adds the forecasting results directly to the input data structure.
         """
-        time_filter_feature = cls.get_time_filter_feature(features.options)
+
+        _options = None
+        for feature in features.features:
+            if _options:
+                if _options != feature.options:
+                    raise ValueError("All features must have the same options.")
+            _options = feature.options
+
+        time_filter_feature = cls.get_time_filter_feature(_options)
 
         cls._check_time_filter_feature_exists(data, time_filter_feature)
         cls._check_time_filter_feature_is_datetime(data, time_filter_feature)
 
-        # Process each requested feature
-        for feature_name in features.get_all_names():
-            algorithm, horizon, time_unit = cls.parse_forecast_prefix(feature_name)
-            mloda_source_feature = FeatureChainParser.extract_source_feature(feature_name, cls.PREFIX_PATTERN)
+        # Store the original clean data
+        original_data = data
 
-            cls._check_source_feature_exists(data, mloda_source_feature)
+        # Collect all results before modifying the data
+        results = []
+
+        # Process each requested feature with the original clean data
+        for feature in features.features:
+            algorithm, horizon, time_unit, mloda_source_feature = cls._extract_forecasting_parameters(feature)
+
+            cls._check_source_feature_exists(original_data, mloda_source_feature)
 
             # Check if we have a trained model in the artifact
             model_artifact = None
@@ -298,20 +339,79 @@ class ForecastingFeatureGroup(AbstractFeatureGroup):
                 if model_artifact is None:
                     raise ValueError("No artifact to load although it was requested.")
 
-            # Perform forecasting
+            # Perform forecasting using the original clean data
             result, updated_artifact = cls._perform_forecasting(
-                data, algorithm, horizon, time_unit, mloda_source_feature, time_filter_feature, model_artifact
+                original_data, algorithm, horizon, time_unit, mloda_source_feature, time_filter_feature, model_artifact
             )
 
             # Save the updated artifact if needed
             if features.artifact_to_save and updated_artifact and not features.artifact_to_load:
                 features.save_artifact = updated_artifact
 
-            # Add the result to the data
+            # Store the result for later addition
+            results.append((feature.get_name(), result))
+
+        # Add all results to the data at once
+        for feature_name, result in results:
             data = cls._add_result_to_data(data, feature_name, result)
 
         return data
 
+    @classmethod
+    def _extract_forecasting_parameters(cls, feature: Feature) -> tuple[str, int, str, str]:
+        """
+        Extract forecasting parameters from a feature.
+
+        Tries string-based parsing first, falls back to configuration-based approach.
+
+        Args:
+            feature: The feature to extract parameters from
+
+        Returns:
+            Tuple of (algorithm, horizon, time_unit, source_feature_name)
+
+        Raises:
+            ValueError: If parameters cannot be extracted
+        """
+        # Try string-based parsing first
+        feature_name_str = feature.name.name if hasattr(feature.name, "name") else str(feature.name)
+
+        if cls.PATTERN in feature_name_str:
+            algorithm, horizon, time_unit = cls.parse_forecast_prefix(feature_name_str)
+            source_feature_name = FeatureChainParser.extract_source_feature(feature_name_str, cls.PREFIX_PATTERN)
+            return algorithm, horizon, time_unit, source_feature_name
+
+        # Fall back to configuration-based approach
+        source_features = feature.options.get_source_features()
+        source_feature = next(iter(source_features))
+        source_feature_name = source_feature.get_name()
+
+        algorithm = feature.options.get(cls.ALGORITHM)
+        horizon = feature.options.get(cls.HORIZON)
+        time_unit = feature.options.get(cls.TIME_UNIT)
+
+        if algorithm is None or horizon is None or time_unit is None or source_feature_name is None:
+            raise ValueError(f"Could not extract forecasting parameters from: {feature.name}")
+
+        # Validate parameters
+        if algorithm not in cls.FORECASTING_ALGORITHMS:
+            raise ValueError(
+                f"Unsupported forecasting algorithm: {algorithm}. "
+                f"Supported algorithms: {', '.join(cls.FORECASTING_ALGORITHMS.keys())}"
+            )
+
+        if time_unit not in cls.TIME_UNITS:
+            raise ValueError(f"Unsupported time unit: {time_unit}. Supported units: {', '.join(cls.TIME_UNITS.keys())}")
+
+        # Convert horizon to integer if it's a string
+        if isinstance(horizon, str):
+            horizon = int(horizon)
+
+        if not isinstance(horizon, int) or horizon <= 0:
+            raise ValueError(f"Invalid horizon: {horizon}. Must be a positive integer.")
+
+        return algorithm, horizon, time_unit, source_feature_name
+
     @classmethod
     def _check_time_filter_feature_exists(cls, data: Any, time_filter_feature: str) -> None:
         """
@@ -396,30 +496,3 @@ class ForecastingFeatureGroup(AbstractFeatureGroup):
             A tuple containing (forecast_result, updated_artifact)
         """
         raise NotImplementedError(f"_perform_forecasting not implemented in {cls.__name__}")
-
-    @classmethod
-    def configurable_feature_chain_parser(cls) -> Optional[Type[FeatureChainParserConfiguration]]:
-        """
-        Returns the FeatureChainParserConfiguration class for this feature group.
-
-        This method allows the Engine to automatically create features with the correct
-        naming convention based on configuration options, rather than requiring explicit
-        feature names.
-
-        Returns:
-            A configured FeatureChainParserConfiguration class
-        """
-        return create_configurable_parser(
-            parse_keys=[
-                cls.ALGORITHM,
-                cls.HORIZON,
-                cls.TIME_UNIT,
-                DefaultOptionKeys.mloda_source_feature,
-            ],
-            feature_name_template="{algorithm}_forecast_{horizon}{time_unit}__{mloda_source_feature}",
-            validation_rules={
-                cls.ALGORITHM: lambda x: x in cls.FORECASTING_ALGORITHMS,
-                cls.TIME_UNIT: lambda x: x in cls.TIME_UNITS,
-                cls.HORIZON: lambda x: (isinstance(x, int) or (isinstance(x, str) and x.isdigit())) and int(x) > 0,
-            },
-        )
